

-- Level Class --
-----------------
local Level = {}
local map
local arrow
local amount = 0
local lastDestination
local Passenger = require("source.Passenger")

local function boxCollision(a, b)
  return a.x + a.w > b.x and a.x < b.x + b.w
  and a.y + a.h > b.y and a.y < b.y + b.h
end


local function newPad(id, x, y, w, h)
  local pad = {}
  pad.x = x
  pad.y = y
  pad.w = w
  pad.h = h
  pad.id = id
  
  function pad:update(dt)
  end
  
  function pad:draw()
    love.graphics.rectangle("line", self.x, self.y, self.w, self.h)
  end
  
  return pad
end


-- Body parts (destruction) --
------------------------------
function Level:newBody(image, x, y, xvel, yvel)
  local body = {}
  body.image = image
  body.w = body.image:getWidth()
  body.h = body.image:getHeight()
  body.x = x
  body.y = y
  body.xvel = xvel
  body.yvel = yvel
  body.r = math.random(0, 100)
  body.rotVel = math.random(-60, 60)
  
  function body:update(dt)
    self.yvel = self.yvel + (Level.gravity*2) * dt
    self.x = self.x + self.xvel * dt
    self.y = self.y + self.yvel * dt
    self.r = self.r + self.rotVel * dt
    if self.y > HEIGHT then self.removed = true end
  end
  
  function body:draw()
    love.graphics.draw(self.image, self.x, self.y,self.r*math.pi/180, 1, 1, self.w/2, self.h / 2)
  end
  
  table.insert(self.bodies, body)
  return body
end


function Level:getMap()
  return map
end


function Level:spawnPassenger()
  local amount = 0
  for i, pad in pairs(self.pads) do
    amount = amount + 1
  end
  local randomStart = math.random(1, amount-1)
  local randomDestination = math.random(1, amount-1)
  if randomStart == randomDestination or randomStart == lastStart or randomDestination == lastDestination then
    self:spawnPassenger()
    return
  end
  lastStart = randomStart
  lastDestination = randomDestination
  
  local passenger = Passenger.new(self.pads[tostring(randomStart)], self.pads[tostring(randomDestination)], math.random(15, 20))
  self.passengers[#self.passengers+1] = passenger
  self.passengerAmount = self.passengerAmount-1
  return passenger
end


function Level:load(id)
  
  self.bodies = {}
  -- Level 1 arrow
  arrow = {}
  arrow.image = love.graphics.newImage("assets/arrow.png")
  arrow.w = arrow.image:getWidth()
  arrow.h = arrow.image:getHeight()
  arrow.x = WIDTH / 2 - arrow.w / 2
  arrow.y = 8
  arrow.timer = 0
  
  self.pads = {}
  self.passengers = {}
  map = sti.new("assets/level"..id)
  self.passengerAmount = map.properties.passengerAmount
  self.gravity = 50
  
  -- Place pads
  for y = 1, map.layers["objects"].height do
    for x = 1, map.layers["objects"].width do
      if map.layers["objects"].data[y][x] then
        local id = map.layers["objects"].data[y][x].gid - map.tilesets[map.layers["objects"].data[y][x].tileset].firstgid
        local padid
        for i, tile in ipairs(map.tilesets[map.layers["objects"].data[y][x].tileset].tiles) do
          if tile.id == id then
            padid = tile.properties["padid"]
          end
        end
        self.pads[padid] = newPad(padid, x*8-8+2, y*8, 48-4, .25)
      end
    end
  end
  return map
end


function Level:update(dt)
  
  -- Spawn passengers
  if #self.passengers == 0 and tonumber(self.passengerAmount) > 0 then
    self:spawnPassenger()
    if tonumber(map.properties.passengerAmount) >= 4 and tonumber(self.passengerAmount) >= 2 and math.random(1, 3) == 3 then
      self:spawnPassenger()
    end
  end
  
  -- Update STI map
  map:update(dt)
  
  if not player.removed then
    
    -- Map collision
    local collision = map:getCollisionMap("level")
    for y = 1, #collision.data do
      for x = 1, #collision.data[1] do
        local tile = {id = collision.data[y][x], x = x * 8 - 8, y = y * 8 - 7, w = 8, h = 8-2}
        for i, hitbox in ipairs(player.hitboxes) do
          if tile.id == 1 and boxCollision(hitbox, tile) then
            player:kill()
            return
          end
        end
      end
    end
  
    -- Exit collision
    collision = map:getCollisionMap("exit")
    if map.layers["exit"].visible then 
      for y = 1, #collision.data do
        for x = 1, #collision.data[1] do
          local tile = {id = collision.data[y][x], x = x * 8 - 8, y = y * 8 - 7, w = 8, h = 8-2}
          for i, hitbox in ipairs(player.hitboxes) do
            if tile.id == 1 and boxCollision(hitbox, tile) then
              player:kill()
              return
            end
          end
        end
      end
    end
 
    -- PADS --
    for i, pad in pairs(self.pads) do
      local p = player.hitboxes[2]
      local gear = player.landingGear
      if boxCollision({x = p.x, y = p.y, w = p.w, h = p.h+1}, pad) and player.velY < 40 and p.x > pad.x and p.x + p.w < pad.x + pad.w and gear.lowered then
        player:setFloor(pad.y)
        player.padId = pad.id
      end
    end
  
  end

 
  -- PASSENGERS
  for i, passenger in ipairs(self.passengers) do
    if not passenger.removed then passenger:update(dt)
    else table.remove(self.passengers, i) end
  end
  
    -- Level completed !
  if #self.passengers == 0 and self.passengerAmount == 0 then
    map.layers["exit"].visible = false
    if player.y < -player.h + 2 and player.x > WIDTH / 2 - 24 and player.x < WIDTH / 2 + 24 then map.completed = true end
  end

  -- Update body parts
  for i, body in ipairs(self.bodies) do
    if not body.removed then body:update(dt)
    else table.remove(self.bodies, i) end
  end
  
end


function Level:draw()
  -- Draw map --
  map:draw()
  
  -- Draw Body parts --
  for i, body in ipairs(self.bodies) do body:draw() end
  
  -- PASSENGERS --
  for i, passenger in ipairs(self.passengers) do
    passenger:draw()
  end
  
  -- Show arrow when finished --
  local dt = love.timer.getDelta()
  if not map.layers["exit"].visible then
    arrow.timer = arrow.timer + dt
    arrow.y = arrow.y + math.sin(arrow.timer * 8) * 20 * dt
    love.graphics.draw(arrow.image, arrow.x, arrow.y)
  end
  
end

return Level