

-- Passenger Class --
---------------------
local Object = require("source.Object")
local Passenger = Object:new()
Passenger.__index = Passenger


local sounds = {
  ["pad1"] = love.audio.newSource("assets/pad1.ogg", "static"),
  ["pad2"] = love.audio.newSource("assets/pad2.ogg", "static"),
  ["pad3"] = love.audio.newSource("assets/pad3.ogg", "static"),
  ["pad4"] = love.audio.newSource("assets/pad4.ogg", "static"),
  ["pad5"] = love.audio.newSource("assets/pad5.ogg", "static"),
  ["pad6"] = love.audio.newSource("assets/pad6.ogg", "static"),
  ["pad7"] = love.audio.newSource("assets/pad7.ogg", "static"),
  ["pad8"] = love.audio.newSource("assets/pad8.ogg", "static"),
  ["pad9"] = love.audio.newSource("assets/pad9.ogg", "static"),
  ["pad10"] = love.audio.newSource("assets/pad10.ogg", "static"),
  hey = love.audio.newSource("assets/heytaxi.ogg", "static"),
  ["thanks1"] = love.audio.newSource("assets/thanks.ogg", "static"),
  ["thanks2"] = love.audio.newSource("assets/thankyou.ogg", "static"),
  whistle = love.audio.newSource("assets/whistle.ogg", "static"),
  }

function Passenger.new(start, destination, money)
  local self = setmetatable({}, Passenger)
  self.start = start
  self.destination = destination
  self.money = money
  self.position = start.id
  self.w = 10
  self.h = 14
  self.speed = 15
  self.x = math.random(start.x, start.x + start.w - self.w)
  self.y = start.y
  self.arrived = false
  self.color = {math.random(200), math.random(200), 0}
  self.mode = "waiting"
  self.timer = 0
  self.spawnTimer = 0
  self.spawnTime = math.random(0, 2)
  self.waitTime = 5
  self.whistlePlayed = false
  self.thanksPlayed = false
  self.targety = self.y - self.h
  self.waveAnim = newAnimation(love.graphics.newImage("assets/passengerAnim.png"), self.w, self.h, .1, 0)
  self.waveAnim:setMode("bounce")
  self.waveAnim:play()
  self.removed = false
  self.pitch = math.random(-100, 600)/ 100 / 12
  
  return self
end


function Passenger:move(xvel, yvel)
  local dt = love.timer.getDelta()
  if xvel then
    self.x = self.x + xvel * dt
  end
  if yvel then
    self.y = self.y + yvel * dt
  end
end


function Passenger:update(dt)
  self.waveAnim:update(dt)
  
  self.spawnTimer = self.spawnTimer + dt
  -- Make sure the player object knows about the current passenger
  player.passenger = self

  -- Decrease money over time
  if self.money > 0 then self.money = self.money - .25 * dt
  else
    self.money = 0
    self.timer = self.timer + dt
    if self.timer > self.waitTime then
      if self.y < self.targety + self.h then self:move(0, self.speed) end
      if self.y > self.targety + self.h then
        self:destroy()
      end
    end
  end

  -- Update money in hud
  if self.position == self.start.id or self.position == "air" then
    player.passengerMoney = self.money
  end
  
  -- Arrived at destination pad
  if player.padId == self.destination.id and self.position ~= self.start.id and not self.arrived then
    self.position = self.destination.id
    self.x = player.x + player.w / 2 - self.w / 2
    self.y = self.destination.y - self.h
    self.targety = self.y
    player.passenger = {}
    player.inUse = false
    player.destination = nil
    player.passengerMoney = nil
    player.money = player.money + math.floor(self.money)
    self.arrived = true
  end
  
  -- Movement --
  --------------
  if self.position == self.start.id then
    -- Move up
    if self.spawnTimer > self.spawnTime then
      if not self.whistlePlayed then
        sounds.whistle:setPitch(1+self.pitch)
        sounds.whistle:play()
        self.whistlePlayed = true
      end
      if self.y > self.targety and self.timer < self.waitTime then self:move(0, -self.speed) end
      if self.y < self.targety then self.y = self.targety end
    end
    
    self.waveAnim:play()
    
    -- Go to taxi and get in
    if self.position == player.padId and self.y == self.targety and not player.inUse then
      self.waveAnim:stop()
      self.waveAnim:seek(8)
      if self.x + self.w / 2 < player.x + player.w / 2 then
        self:move(self.speed, 0)
        if  self.x + self.w / 2 > player.x + player.w / 2 then
          player.inUse = true
          player.destination = self.destination.id
          self.position = "air"
          sounds["pad"..self.destination.id]:setPitch(1+self.pitch)
          sounds["pad"..self.destination.id]:play()
        end
      elseif self.x + self.w / 2 > player.x + player.w / 2 then
        self:move(-self.speed, 0)
        if  self.x + self.w / 2 < player.x + player.w / 2 then
          player.inUse = true
          player.destination = self.destination.id
          self.position = "air"
          sounds["pad"..self.destination.id]:setPitch(1+self.pitch)
          sounds["pad"..self.destination.id]:play()
        end
      end
    end
    
    -- Get out
  elseif self.position == self.destination.id then
    if not self.thanksPlayed then
      local rnd = math.random(1, 4)
      if rnd < 3 then
        sounds["thanks"..rnd]:setPitch(1+self.pitch)
        sounds["thanks"..rnd]:play()
      end
    self.thanksPlayed = true
    end
    
      if self.x + self.w / 2 <= self.destination.x + self.destination.w then self:move(self.speed, 0) end
      if self.x + self.w / 2 > self.destination.x + self.destination.w - self.w then
        self.x = self.destination.x + self.destination.w - self.w
        if self.y < self.targety + self.h then self:move(0, self.speed) end
        if self.y > self.targety + self.h then
          self:destroy()
        end
      end
    end
    
end


function Passenger:draw()
  love.graphics.setScissor(self.x*SCALEX, self.targety*SCALEY, self.w*SCALEX, self.h*SCALEY)
  if self.position ~= "air" then
    self.waveAnim:draw(self.x, self.y)
  end
  love.graphics.setScissor()
end

return Passenger
