
-- // Taxi \\ --
----------------
local Object = require("source.Object")
local Taxi = Object:new()
Taxi.__index = Taxi

-- Dependencies --
------------------
local Level = require("source.Level")
local SceneManager = require("source.SceneManager")

-- Declarations --
------------------
local keyboard = love.keyboard
local passengerBody = love.graphics.newImage("assets/passengerBody.png")
local taxiBody = {
  love.graphics.newImage("assets/taxiBody1.png"),
  love.graphics.newImage("assets/taxiBody2.png"),
  love.graphics.newImage("assets/taxiBody3.png"),
  }

local sounds = {
  thrust = love.audio.newSource("assets/thrust.ogg", "static"),
  explosion = love.audio.newSource("assets/explosion.ogg", "static"),
  fuel = love.audio.newSource("assets/nofuel.ogg", "static"),
  noo = love.audio.newSource("assets/noo.ogg", "static")
}

function Taxi:new(x, y)
  local self = setmetatable({}, Taxi)
  self.image = love.graphics.newImage("assets/taxi.png")
  self.x = x or 0
  self.y = y or 0
  self.w = self.image:getWidth()
  self.h = self.image:getHeight()
  self.accX = 75
  self.accY = 130
  self.state = "flying"
  self.flipped = false
  self.money = 0
  self.lives = 5
  self.inUse = false
  self.gravity = Level.gravity
  self.velX = 0
  self.velY = 0
  self.fuel = 100
  self.padId = ""
  self.consumption = 2.75
  self.hitboxes = {}
  self.floor = HEIGHT - 16
  
  self.flameImage = love.graphics.newImage("assets/flame.png")
  local iw, ih = self.flameImage:getWidth(), self.flameImage:getHeight()
  self.flames = {
    [1] = love.graphics.newQuad(0, 0, 3-.1, 3-.1, iw, ih),
    [2] = love.graphics.newQuad(3, 0, 3-.1, 3-.1, iw, ih),
    [3] = love.graphics.newQuad(6, 0, 3-.1, 3-.1, iw, ih),
    [4] = love.graphics.newQuad(9, 0, 3-.1, 3-.1, iw, ih),
  }
  self.currentFlame = 0
  
  self.landingGear = {
    image = love.graphics.newImage("assets/landinggear.png"),
    x = self.x,
    y = self.y+self.h,
    lowered = true
  }
  
  return self
end


function Taxi:getHitbox(index)
  return self.hitboxes[index]
end


function Taxi:setFloor(v)
  self.floor = v
end


function Taxi:setConsumption(v)
  self.consumption = v 
end


function Taxi:respawn(x, y)
  self:setPosition(x, y)
  self.landingGear.lowered = true
  self.velY = 0
  self.inUse = false
end


function Taxi:kill()
  self.lives = self.lives - 1
  
  -- Passenger --
  ---------------
  if self.inUse and self.passenger.position == "air" or self.inUse and self.passenger.position == self.passenger.start.id then
    Level.passengerAmount = Level.passengerAmount + 1
    Level:spawnPassenger()
    local pitch = self.passenger.pitch
    sounds.noo:setPitch(1+pitch)
    sounds.noo:play()
    Level:newBody(passengerBody, self.x + self.w / 2, self.y + self.h / 2, self.velX + math.random(-30, 30), self.velY - 60)
    self.passenger:destroy()
    self.destination = nil
  end
  
  -- Taxi --
  ----------
  Level:newBody(taxiBody[1], self.x, self.y + self.h / 2, math.random(-50, 50), -50)
  Level:newBody(taxiBody[2], self.x + self.w / 2, self.y + self.h / 2, math.random(-50, 50), -60)
  Level:newBody(taxiBody[3], self.x + self.w, self.y + self.h / 2, math.random(-50, 50), -40)
  self:respawn(Level.pads["Gas Station"].x + Level.pads["Gas Station"].w / 2 - 16, Level.pads["Gas Station"].y - 10)
  sounds.explosion:play()
  self.fuel = 100
  self.money = 0
  
  if self.lives < 1 then 
    sounds.thrust:stop()
    self:destroy()
    SceneManager.getArray("GameScene"):enter()
  end
  
end
 

function Taxi:move(xvel, yvel)
  local dt = love.timer.getDelta()
  if x ~= 0 then self.velX = self.velX + xvel * dt end
  if y ~= 0 then self.velY = self.velY + yvel * dt end
  self.fuel = self.fuel - self.consumption * dt
  self.padId = ""
end


function Taxi:update(dt)
  -- Update gear
  local gear = self.landingGear
  gear.x = self.x + 4
  gear.y = self.y+self.h
  
  -- Refill gas tank
  if self.padId == "Gas Station" then
    if self.fuel < 100 and self.money > 0 then
      self.fuel = self.fuel + 10 * dt
      self.money = self.money - 5 * dt
      if self.money < 0 then self.money = 0 end
      if self.fuel > 100 then self.fuel = 100 end
    end
  end
  
  -- Update hitboxes
  self.hitboxes = {
    [1] = {x = self.x + 2, y = self.y, w = self.w / 2, h = self.h / 2},
    [2] = {x = self.x, y = self.y + self.h / 2, w = self.w, h = self.h / 2}, }
  
  if self.flipped then self.hitboxes[1].x = self.x + 12 end
  if gear.lowered then self.hitboxes[2].h = self.h / 2 + gear.image:getHeight() end
  
  -- Shortcuts for key input --
  local up, down, left, right = keyboard.isDown("up"), keyboard.isDown("down"), keyboard.isDown("left"), keyboard.isDown("right")
  
  -- SOUNDS --
  ------------
  -- any direction then play thrust sound
    if left or right or up or down then
      if self.fuel > 0 and not sounds.thrust:isPlaying() and self.state == "flying" then
        sounds.thrust:play()
      end
    else
      sounds.thrust:pause()
    end

  if self.fuel < 20 and not sounds.fuel:isPlaying() then
    sounds.fuel:play()
  end
  
  -- Update velocity / gravity --
  if self.state ~= "landed" or gear.lowered then self.velY = self.velY + self.gravity * dt end
  
  -- Apply velocities to the taxi position --
  self.x = self.x + self.velX * dt
  self.y = self.y + self.velY * dt
  
  -- Bind keys and calculate acclaration --
  if self.fuel > 0 then
    -- X Axis --
    ------------
    if not gear.lowered then
      if left and self.velX > - 150 then
        self:move(-self.accX, 0)
        self.flipped = true
        self.currentFlame = 2
      end
      if right and self.velX < 150 then
        self:move(self.accX, 0)
        self.flipped = false
        self.currentFlame = 1
      end
    else
      if self.velX > 0 then self.velX = self.velX - 15 * dt
      elseif self.velX < 0 then self.velX = self.velX + 15 * dt end
    end
    
    -- Y Axis --
    ------------
    if up then self:move(0, -self.accY) ; self.currentFlame = 4 end
    if down then self:move(0, self.accY/2) ; self.currentFlame = 3 end
    
    -- Teleport --
    if self.x > WIDTH then self.x = -self.w end
    if self.x < -self.w then self.x = WIDTH end
    if self.y > HEIGHT - 18 then self.y = -self.h end
    if self.y < -self.h then self.y = HEIGHT - 18 end
    
  end
  
  -- Setup boundaries --
  if self.y + (self.h+gear.image:getHeight()) >= self.floor then
    self.y = self.floor - (self.h+gear.image:getHeight())
    self.state = "landed"
    self.velX = 0
    if not up then self.velY = 0 end
  else
    self.state = "flying"
  end
    self.floor = HEIGHT
    
end


function Taxi:draw()
  -- Flip --
  ----------
  love.graphics.setScissor(0, 0, WIDTH*SCALEX, HEIGHT*SCALEY-64)
  if self.flipped then love.graphics.draw(self.image, self.x, self.y, 0, -1, 1, self.w, 0)
  else love.graphics.draw(self.image, self.x, self.y) end

  -- Gear --
  ----------
  local gear = self.landingGear
  if gear.lowered then love.graphics.draw(gear.image, gear.x, gear.y) end
  
  
  -- Flames --
  if self.currentFlame == 1 then love.graphics.draw(self.flameImage, self.flames[1], self.x-3, self.y + self.h / 2-1)
  elseif self.currentFlame == 2 then love.graphics.draw(self.flameImage, self.flames[2], self.x + self.w, self.y + self.h / 2-1)
  elseif self.currentFlame == 3 then
    if self.flipped then love.graphics.draw(self.flameImage, self.flames[4], self.x + self.w / 2 + 2, self.y-3)
    else love.graphics.draw(self.flameImage, self.flames[4], self.x + self.w / 2 - 4, self.y-3) end
  elseif self.currentFlame == 4 then love.graphics.draw(self.flameImage, self.flames[3], self.x + self.w / 2, self.y + self.h)
  end
  
  -- Reset flames
  self.currentFlame = 0
  love.graphics.setScissor()
end


function Taxi:keypressed(key)
  local gear = self.landingGear
  if key == " " then
    gear.lowered = not gear.lowered
  end
end

return Taxi
